@@ -62,8 +62,6 @@ Nepomuk2::IndexFolderSelectionDialog::IndexFolderSelectionDialog( QWidget* paren
     setupUi( mainWidget() );
     setCaption( i18nc("@title:window Referring to the folders which will be searched for files to index for desktop search", "Customizing Index Folders"));
 
-    m_editExcludeFilters->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-
     m_folderModel = new FolderSelectionModel( m_viewIndexFolders );
     m_viewIndexFolders->setModel( m_folderModel );
     m_viewIndexFolders->setHeaderHidden( true );
@@ -99,18 +97,13 @@ void Nepomuk2::IndexFolderSelectionDialog::setFolders( const QStringList& includ
     }
 
     // make sure that the tree is expanded to show all selected items
+    // expand the parent of each folder, as there is no point expanding the the folder itself
     foreach( const QString& dir, m_folderModel->includeFolders() + m_folderModel->excludeFolders() ) {
-        expandRecursively( m_folderModel->index( dir ), m_viewIndexFolders );
+        expandRecursively( m_folderModel->index( dir ).parent(), m_viewIndexFolders );
     }
 }
 
 
-void Nepomuk2::IndexFolderSelectionDialog::setExcludeFilters( const QStringList& filters )
-{
-    m_editExcludeFilters->setItems( filters );
-}
-
-
 void Nepomuk2::IndexFolderSelectionDialog::setIndexHiddenFolders( bool enable )
 {
     m_checkShowHiddenFolders->setChecked( enable );
@@ -139,12 +132,6 @@ QStringList Nepomuk2::IndexFolderSelectionDialog::excludeFolders() const
 }
 
 
-QStringList Nepomuk2::IndexFolderSelectionDialog::excludeFilters() const
-{
-    return m_editExcludeFilters->items();
-}
-
-
 bool Nepomuk2::IndexFolderSelectionDialog::indexHiddenFolders() const
 {
     return m_checkShowHiddenFolders->isChecked();
